@@ -11,6 +11,7 @@
 #include "session.h"
 #include "util.h"
 
+#include <cstdlib>
 #include <fstream>
 #include <sstream>
 
@@ -41,15 +42,18 @@ compile_pass (systemtap_session& s)
 
   // Clever hacks copied from vmware modules
   o << "stap_check_gcc = $(shell if $(CC) $(1) -S -o /dev/null -xc /dev/null > /dev/null 2>&1; then echo \"$(1)\"; else echo \"$(2)\"; fi)" << endl;
-  o << "stap_check_build = " /* << "set -x; " */ << "$(shell if $(CC) $(CPPFLAGS) $(CFLAGS_KERNEL) $(EXTRA_CFLAGS) -DKBUILD_BASENAME=\\\"" << s.module_name << "\\\" -Werror -S -o /dev/null -xc $(1) > /dev/null 2>&1; then echo \"$(2)\"; else echo \"$(3)\"; fi)" << endl;
+  o << "stap_check_build = $(shell " << "set -x; " << " if $(CC) $(CPPFLAGS) $(CFLAGS_KERNEL) $(EXTRA_CFLAGS) $(CFLAGS) -DKBUILD_BASENAME=\\\"" << s.module_name << "\\\" -Werror -S -o /dev/null -xc $(1) > /dev/null ; then echo \"$(2)\"; else echo \"$(3)\"; fi)" << endl;
+
 
   o << "SYSTEMTAP_RUNTIME = \"" << s.runtime_path << "\"" << endl;
 
   // "autoconf" options go here
 
   // enum hrtimer_mode renaming near 2.6.21; see tapsets.cxx hrtimer_derived_probe_group::emit_module_decls
-  o << "CFLAGS += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-hrtimer-rel.c, -DSTAPCONF_HRTIMER_REL,)" << endl;
-
+  string module_cflags = "CFLAGS_" + s.module_name + ".o";
+  o << module_cflags << " :=" << endl;
+  o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-hrtimer-rel.c, -DSTAPCONF_HRTIMER_REL,)" << endl;
+  o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-inode-private.c, -DSTAPCONF_INODE_PRIVATE,)" << endl;
 
   for (unsigned i=0; i<s.macros.size(); i++)
     o << "CFLAGS += -D " << lex_cast_qstring(s.macros[i]) << endl;
@@ -89,11 +93,23 @@ compile_pass (systemtap_session& s)
 	return rc;
     }  
 
+  // Before running make, fix up the environment a bit.  PATH should
+  // already be overridden.  Clean out a few variables that
+  // /lib/modules/${KVER}/build/Makefile uses.
+  rc = unsetenv("ARCH") || unsetenv("KBUILD_EXTMOD")
+      || unsetenv("CROSS_COMPILE") || unsetenv("KBUILD_IMAGE")
+      || unsetenv("KCONFIG_CONFIG") || unsetenv("INSTALL_PATH");
+  if (rc)
+    {
+      const char* e = strerror (errno);
+      cerr << "unsetenv failed: " << e << endl;
+    }
+
   // Run make
   string make_cmd = string("make")
     + string (" -C \"") + module_dir + string("\"");
   make_cmd += string(" M=\"") + s.tmpdir + string("\" modules");
-  
+
   if (s.verbose > 1)
     make_cmd += " V=1";
   else
@@ -118,6 +134,7 @@ run_pass (systemtap_session& s)
   string staprun_cmd = string("sudo ") 
     + string(BINDIR) + "/staprun "
     + (s.verbose>1 ? "-v " : "")
+    + (s.verbose>2 ? "-v " : "")
     + "-u " + username + " "
     + (s.output_file.empty() ? "" : "-o " + s.output_file + " ");
   
