@@ -62,8 +62,50 @@ derived_probe::derived_probe (probe *p, probe_point *l):
 }
 
 
+void
+derived_probe::printsig (ostream& o) const
+{
+  probe::printsig (o);
+  printsig_nested (o);
+}
+
+void
+derived_probe::printsig_nested (ostream& o) const
+{
+  // We'd like to enclose the probe derivation chain in a /* */
+  // comment delimiter.  But just printing /* base->printsig() */ is
+  // not enough, since base might itself be a derived_probe.  So we,
+  // er, "cleverly" encode our nesting state as a formatting flag for
+  // the ostream.
+  ios::fmtflags f = o.flags (ios::internal);
+  if (f & ios::internal)
+    {
+      // already nested
+      o << " <- ";
+      base->printsig (o);
+    }
+  else
+    {
+      // outermost nesting
+      o << " /* <- ";
+      base->printsig (o);
+      o << " */";
+    }
+  // restore flags
+  (void) o.flags (f);
+}
+
+
+void
+derived_probe::collect_derivation_chain (std::vector<derived_probe*> &probes_list)
+{
+  probes_list.push_back(this);
+  base->collect_derivation_chain(probes_list);
+}
+
+
 probe_point*
-derived_probe::sole_location ()
+derived_probe::sole_location () const
 {
   if (locations.size() == 0)
     throw semantic_error ("derived_probe with no locations", this->tok);
@@ -112,6 +154,15 @@ derived_probe_builder::get_param (std::map<std::string, literal*> const & params
 }
 
 
+bool
+derived_probe_builder::has_null_param (std::map<std::string, literal*> const & params,
+                                       const std::string& key)
+{
+  map<string, literal *>::const_iterator i = params.find(key);
+  return (i != params.end() && i->second == NULL);
+}
+
+
 
 // ------------------------------------------------------------------------
 // Members of match_key.
@@ -381,9 +432,9 @@ alias_expansion_builder
     // The new probe gets the location list of the alias,
     n->locations = alias->locations;
   
-    // the token location of the use,
-    n->tok = use->tok;
-    n->body->tok = use->tok;
+    // the token location of the alias,
+    n->tok = location->tok;
+    n->body->tok = location->tok;
 
     // and statements representing the concatenation of the alias'
     // body with the use's. 
@@ -970,6 +1021,7 @@ systemtap_session::systemtap_session ():
   user_file (0),
   be_derived_probes(0), 
   dwarf_derived_probes(0), 
+  uprobe_derived_probes(0), 
   timer_derived_probes(0), 
   profile_derived_probes(0), 
   mark_derived_probes(0), 
@@ -1323,6 +1375,9 @@ void semantic_pass_opt1 (systemtap_session& s, bool& relaxed_p)
           if (s.verbose>2)
             clog << "Eliding unused function " << s.functions[i]->name
                  << endl;
+	  if (s.tapset_compile_coverage) {
+	    s.unused_functions.push_back (s.functions[i]);
+	  }
           s.functions.erase (s.functions.begin() + i);
           relaxed_p = false;
           // NB: don't increment i
@@ -1363,6 +1418,10 @@ void semantic_pass_opt2 (systemtap_session& s, bool& relaxed_p)
             if (s.verbose>2)
               clog << "Eliding unused local variable "
                    << l->name << " in " << s.probes[i]->name << endl;
+	    if (s.tapset_compile_coverage) {
+	      s.probes[i]->unused_locals.push_back
+		      (s.probes[i]->locals[j]);
+	    }
             s.probes[i]->locals.erase(s.probes[i]->locals.begin() + j);
             relaxed_p = false;
             // don't increment j
@@ -1381,6 +1440,10 @@ void semantic_pass_opt2 (systemtap_session& s, bool& relaxed_p)
               clog << "Eliding unused local variable "
                    << l->name << " in function " << s.functions[i]->name
                    << endl;
+	    if (s.tapset_compile_coverage) {
+              s.functions[i]->unused_locals.push_back
+		    (s.functions[i]->locals[j]);
+	    }
             s.functions[i]->locals.erase(s.functions[i]->locals.begin() + j);
             relaxed_p = false;
             // don't increment j
@@ -1397,6 +1460,9 @@ void semantic_pass_opt2 (systemtap_session& s, bool& relaxed_p)
           if (s.verbose>2)
             clog << "Eliding unused global variable "
                  << l->name << endl;
+	  if (s.tapset_compile_coverage) {
+            s.unused_globals.push_back(s.globals[i]);
+	  }
           s.globals.erase(s.globals.begin() + i);
           relaxed_p = false;
           // don't increment i
@@ -1453,11 +1519,22 @@ dead_assignment_remover::visit_assignment (assignment* e)
       // clog << "Checking assignment to " << leftvar->name << " at " << *e->tok << endl;
       if (vut.read.find(leftvar) == vut.read.end()) // var never read?
         {
-          if (session.verbose>2)
-            clog << "Eliding assignment to " << leftvar->name 
-                 << " at " << *e->tok << endl;
-          *current_expr = e->right; // goodbye assignment*
-          relaxed_p = false;
+          // NB: Not so fast!  The left side could be an array whose
+          // index expressions may have side-effects.  This would be
+          // OK if we could replace the array assignment with a 
+          // statement-expression containing all the index expressions
+          // and the rvalue... but we can't.
+
+          varuse_collecting_visitor vut;
+          e->left->visit (& vut);
+          if (vut.side_effect_free ()) // XXX: use _wrt() once we track focal_vars
+            {
+              if (session.verbose>2)
+                clog << "Eliding assignment to " << leftvar->name 
+                     << " at " << *e->tok << endl;
+              *current_expr = e->right; // goodbye assignment*
+              relaxed_p = false;
+            }
         }
     }
 }
@@ -1537,23 +1614,7 @@ dead_stmtexpr_remover::visit_expr_statement (expr_statement *s)
 
   varuse_collecting_visitor vut;
   s->value->visit (& vut);
-
-  // Note that side-effect-freeness is not simply this test:
-  //
-  // (vut.written.empty() && !vut.embedded_seen)
-  //
-  // That's because the vut.written list may consist of local
-  // variables of called functions.  Visible side-effects occur if
-  // *our* locals, or any *globals* are written-to.
-
-
-  set<vardecl*> intersection;
-  insert_iterator<set<vardecl*> > int_it (intersection, intersection.begin());
-  set_intersection (vut.written.begin(), vut.written.end(),
-                    focal_vars.begin(), focal_vars.end(),
-                    int_it);
-
-  if (intersection.empty() && ! vut.embedded_seen)
+  if (vut.side_effect_free_wrt (focal_vars))
     {
       if (session.verbose>2)
         clog << "Eliding side-effect-free expression "
@@ -1569,15 +1630,6 @@ dead_stmtexpr_remover::visit_expr_statement (expr_statement *s)
       
       relaxed_p = false;
     }
-  else if(session.verbose>3)
-    {
-      clog << "keeping expression " << *s->tok 
-           << " because it writes: ";
-      for (std::set<vardecl*>::iterator k = intersection.begin();
-           k != intersection.end(); k++)
-        clog << (*k)->name << " ";
-      clog << "and/or embedded: " << vut.embedded_seen << endl;
-    }  
 }
 
 
@@ -2700,11 +2752,18 @@ typeresolution_info::invalid (const token* tok, exp_type pe)
       string nm = (current_function ? current_function->name :
                    current_probe ? current_probe->name :
                    "?");
-      cerr << "semantic error: " + nm + " with invalid type " << pe << " for ";
-      if (tok)
-        cerr << *tok;
+      if (tok && tok->type == tok_operator)
+        {
+	  cerr << "semantic error: " + nm + " uses invalid " << *tok;
+        }
       else
-        cerr << "a token";
+        {
+	  cerr << "semantic error: " + nm + " with invalid type " << pe << " for ";
+	  if (tok)
+	      cerr << *tok;
+	  else
+	      cerr << "a token";
+	}
       cerr << endl;
     }
 }
