@@ -115,16 +115,26 @@ struct derived_probe: public probe
   virtual probe* basest () { return base->basest(); }
   virtual ~derived_probe () {}
   virtual void join_group (systemtap_session& s) = 0;
-  virtual probe_point* sole_location ();
+  virtual probe_point* sole_location () const;
+  virtual void printsig (std::ostream &o) const;
+  void printsig_nested (std::ostream &o) const;
+  virtual void collect_derivation_chain (std::vector<derived_probe*> &probes_list);
 
   virtual void emit_probe_context_vars (translator_output*) {}
   // From within unparser::emit_common_header, add any extra variables
   // to this probe's context locals.
 
+  virtual void initialize_probe_context_vars (translator_output*) {}
+  // From within unparser::emit_probe, initialized any extra variables
+  // in this probe's context locals.
+
 public:
   static void emit_common_header (translator_output* o);
   // from c_unparser::emit_common_header
   // XXX: probably can move this stuff to a probe_group::emit_module_decls
+
+  virtual bool needs_global_locks () { return true; }
+  // by default, probes need locks around global variables
 };
 
 // ------------------------------------------------------------------------
@@ -172,6 +182,8 @@ struct derived_probe_builder
   virtual ~derived_probe_builder() {}
   virtual void build_no_more (systemtap_session &) {}
 
+  static bool has_null_param (std::map<std::string, literal*> const & parameters,
+                              const std::string& key);
   static bool get_param (std::map<std::string, literal*> const & parameters,
                          const std::string& key, std::string& value);
   static bool get_param (std::map<std::string, literal*> const & parameters,
