@@ -1017,24 +1017,28 @@ c_translate_location (struct obstack *pool,
 /* Emit "uintNN_t TARGET = ...;".  */
 static bool
 emit_base_fetch (struct obstack *pool, Dwarf_Word byte_size,
-		 const char *target, bool decl, struct location *loc)
+                 bool signed_p, const char *target, struct location *loc)
 {
-  if (decl)
-    obstack_printf (pool, "uint%" PRIu64 "_t ", byte_size * 8);
+  bool deref = false;
+  /* int i; */
+
+  /* Emit size/signed coercion. */ 
+  obstack_printf (pool, "{ ");
+  obstack_printf (pool, "%sint%u_t value = ", 
+                  (signed_p ? "" : "u"), (unsigned)(byte_size * 8));
 
   switch (loc->type)
     {
     case loc_address:
       if (byte_size != 0 && byte_size != (Dwarf_Word) -1)
-	obstack_printf (pool, "%s = deref (%" PRIu64 ", addr);",
-			target, byte_size);
+	obstack_printf (pool, "deref (%" PRIu64 ", addr);", byte_size);
       else
-	obstack_printf (pool, "%s = deref (sizeof %s, addr);",
-			target, target);
-      return true;
+	obstack_printf (pool, "deref (sizeof %s, addr);", target);
+      deref = true;
+      break;
 
     case loc_register:
-      obstack_printf (pool, "%s = fetch_register (%u);", target, loc->regno);
+      obstack_printf (pool, "fetch_register (%u);", loc->regno);
       break;
 
     case loc_noncontiguous:
@@ -1046,7 +1050,9 @@ emit_base_fetch (struct obstack *pool, Dwarf_Word byte_size,
       break;
     }
 
-  return false;
+  obstack_printf (pool, "%s = value; ", target);
+  obstack_printf (pool, "}");
+  return deref;
 }
 
 /* Emit "... = RVALUE;".  */
@@ -1231,6 +1237,31 @@ base_byte_size (Dwarf_Die *typedie, struct location *origin)
   return -1;
 }
 
+static Dwarf_Word
+base_encoding (Dwarf_Die *typedie, struct location *origin)
+{
+  if (! (dwarf_tag (typedie) == DW_TAG_base_type ||
+         dwarf_tag (typedie) == DW_TAG_enumeration_type))
+    return -1;
+
+  Dwarf_Attribute attr_mem;
+  Dwarf_Word encoding;
+  if (dwarf_attr_integrate (typedie, DW_AT_encoding, &attr_mem) != NULL
+      && dwarf_formudata (&attr_mem, &encoding) == 0)
+    return encoding;
+
+  (void) origin;
+  /*
+  FAIL (origin,
+	 N_("cannot get encoding attribute for type %s: %s"),
+	 dwarf_diename (typedie) ?: "<anonymous>",
+	 dwarf_errmsg (-1));
+  */
+  return -1;
+}
+
+
+
 /* Fetch the bitfield parameters.  */
 static void
 get_bitfield (struct location *loc,
@@ -1248,7 +1279,7 @@ get_bitfield (struct location *loc,
    at the *INPUT location and store it in lvalue TARGET.  */
 static void
 translate_base_fetch (struct obstack *pool, int indent, Dwarf_Word byte_size,
-		      struct location **input, const char *target)
+                      bool signed_p, struct location **input, const char *target)
 {
   bool deref = false;
 
@@ -1269,7 +1300,8 @@ translate_base_fetch (struct obstack *pool, int indent, Dwarf_Word byte_size,
 	  *input = newp;
 
 	  snprintf (&piece[sizeof "u.pieces.p" - 1], 20, "%" PRIu64, offset);
-	  translate_base_fetch (pool, indent, p->byte_size, input, piece);
+	  translate_base_fetch (pool, indent, p->byte_size, signed_p /* ? */,
+                                input, piece);
 	  (*input)->type = loc_fragment;
 
 	  offset += p->byte_size;
@@ -1287,7 +1319,7 @@ translate_base_fetch (struct obstack *pool, int indent, Dwarf_Word byte_size,
       case 4:
       case 8:
 	obstack_printf (pool, "%*s", indent * 2, "");
-	deref = emit_base_fetch (pool, byte_size, target, false, *input);
+	deref = emit_base_fetch (pool, byte_size, signed_p, target, *input);
 	obstack_printf (pool, "\n");
 	break;
 
@@ -1334,6 +1366,14 @@ c_translate_fetch (struct obstack *pool, int indent,
       || dwarf_formudata (&size_attr, &byte_size) != 0)
     byte_size = base_byte_size (typedie, *input);
 
+  Dwarf_Attribute encoding_attr;
+  Dwarf_Word encoding;
+  if (dwarf_attr_integrate (die, DW_AT_encoding, &encoding_attr) == NULL
+      || dwarf_formudata (&encoding_attr, &encoding) != 0)
+    encoding = base_encoding (typedie, *input);
+  bool signed_p = (encoding == DW_ATE_signed 
+                   || encoding == DW_ATE_signed_char);
+
   *input = discontiguify (pool, indent, *input, byte_size,
 			  max_fetch_size (*input, die));
 
@@ -1342,7 +1382,7 @@ c_translate_fetch (struct obstack *pool, int indent,
       /* This is a bit field.  Fetch the containing base type into a
 	 temporary variable.  */
 
-      translate_base_fetch (pool, indent, byte_size, input, "tmp");
+      translate_base_fetch (pool, indent, byte_size, signed_p, input, "tmp");
       (*input)->type = loc_fragment;
       (*input)->address.declare = "tmp";
 
@@ -1359,7 +1399,7 @@ c_translate_fetch (struct obstack *pool, int indent,
       *input = loc;
     }
   else
-    translate_base_fetch (pool, indent, byte_size, input, target);
+    translate_base_fetch (pool, indent, byte_size, signed_p, input, target);
 }
 
 /* Translate a fragment to store RVALUE into the base-type value of
@@ -1385,7 +1425,7 @@ translate_base_store (struct obstack *pool, int indent, Dwarf_Word byte_size,
       char piece[sizeof "u.pieces.p" + 20] = "u.pieces.p";
       struct location *p;
       for (p = store_loc->pieces; p != NULL; p = p->next)
-	{
+        {
 	  struct location *newp = obstack_alloc (pool, sizeof *newp);
 	  *newp = *p;
 	  newp->next = NULL;
@@ -1445,6 +1485,14 @@ c_translate_store (struct obstack *pool, int indent,
       || dwarf_formudata (&size_attr, &byte_size) != 0)
     byte_size = base_byte_size (typedie, *input);
 
+  Dwarf_Attribute encoding_attr;
+  Dwarf_Word encoding;
+  if (dwarf_attr_integrate (die, DW_AT_encoding, &encoding_attr) == NULL
+      || dwarf_formudata (&encoding_attr, &encoding) != 0)
+    encoding = base_encoding (typedie, *input);
+  bool signed_p = (encoding == DW_ATE_signed 
+                   || encoding == DW_ATE_signed_char);
+
   *input = discontiguify (pool, indent, *input, byte_size,
 			  max_fetch_size (*input, die));
 
@@ -1455,7 +1503,7 @@ c_translate_store (struct obstack *pool, int indent,
       /* This is a bit field.  Fetch the containing base type into a
 	 temporary variable.  */
 
-      translate_base_fetch (pool, indent, byte_size, input, "tmp");
+      translate_base_fetch (pool, indent, byte_size, signed_p, input, "tmp");
       (*input)->type = loc_fragment;
       (*input)->address.declare = "tmp";
 
@@ -1502,7 +1550,9 @@ c_translate_pointer (struct obstack *pool, int indent,
 	  dwarf_diename (typedie) ?: "<anonymous>",
 	  dwarf_errmsg (-1));
 
-  translate_base_fetch (pool, indent + 1, byte_size, input, "addr");
+  bool signed_p = false; /* XXX: Does not matter? */
+
+  translate_base_fetch (pool, indent + 1, byte_size, signed_p, input, "addr");
   (*input)->type = loc_address;
 }
 
