@@ -18,6 +18,7 @@
 #include "hash.h"
 #include "cache.h"
 #include "util.h"
+#include "coveragedb.h"
 
 #include <iostream>
 #include <fstream>
@@ -44,8 +45,8 @@ version ()
 {
   clog
     << "SystemTap translator/driver "
-    << "(version " << VERSION << " built " << DATE << ")" << endl
-    << "(Using " << dwfl_version (NULL) << " libraries.)" << endl
+    << "(version " << VERSION << "/" << dwfl_version (NULL)
+    << " built " << DATE << ")" << endl
     << "Copyright (C) 2005-2007 Red Hat, Inc. and others" << endl
     << "This is free software; see the source for copying conditions." << endl;
 }
@@ -90,7 +91,7 @@ usage (systemtap_session& s, int exitcode)
     << "   -D NM=VAL  emit macro definition into generated C code" << endl
     << "   -R DIR     look in DIR for runtime, instead of" << endl
     <<      "              " << s.runtime_path << endl
-    << "   -r RELEASE use kernel RELEASE, instead of "
+    << "   -r RELEASE cross-compile to kernel RELEASE, instead of "
     << s.kernel_release << endl
     << "   -m MODULE  set probe module name, instead of "
     << s.module_name << endl
@@ -98,7 +99,9 @@ usage (systemtap_session& s, int exitcode)
     << "   -c CMD     start the probes, run CMD, and exit when it finishes"
     << endl
     << "   -x PID     sets target() to PID" << endl
-    << "   -t         benchmarking timing information generated" << endl
+    << "   -t         collect probe timing information" << endl
+    << "   -q         generate information on tapset coverage"
+    << endl
     ;
   // -d: dump safety-related external references
 
@@ -216,6 +219,7 @@ main (int argc, char * const argv [])
   s.perfmon=0;
   s.symtab = false;
   s.use_cache = true;
+  s.tapset_compile_coverage = false;
 
   const char* s_p = getenv ("SYSTEMTAP_TAPSET");
   if (s_p != NULL)  
@@ -262,10 +266,14 @@ main (int argc, char * const argv [])
 	}
     }
 
+  const char* s_tc = getenv ("SYSTEMTAP_COVERAGE");
+  if (s_tc != NULL)
+    s.tapset_compile_coverage = true;
+
   while (true)
     {
       // NB: also see find_hash(), help(), switch stmt below, stap.1 man page
-      int grc = getopt (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:u");
+      int grc = getopt (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:uq");
       if (grc < 0)
         break;
       switch (grc)
@@ -320,6 +328,38 @@ main (int argc, char * const argv [])
 
         case 'm':
           s.module_name = string (optarg);
+	  {
+	    string::size_type len = s.module_name.length();
+
+	    // If the module name ends with '.ko', chop it off since
+	    // modutils doesn't like modules named 'foo.ko.ko'.
+	    if (len > 3 && s.module_name.substr(len - 3, 3) == ".ko")
+	      {
+		s.module_name.erase(len - 3);
+		len -= 3;
+		cerr << "Truncating module name to '" << s.module_name
+		     << "'" << endl;
+	      }
+
+	    // Make sure an empty module name wasn't specified (-m "")
+	    if (len == 0)
+	    {
+		cerr << "Module name cannot be empty." << endl;
+		usage (s, 1);
+	    }
+
+	    // Make sure the module name is only composed of the
+	    // following chars: [_a-zA-Z0-9]
+	    const string identchars("_" "abcdefghijklmnopqrstuvwxyz"
+				    "ABCDEFGHIJKLMNOPQRSTUVWXYZ" "0123456789");
+	    if (s.module_name.find_first_not_of(identchars) != string::npos)
+	      {
+		cerr << "Invalid module name (must only be composed of"
+		    " characters [_a-zA-Z0-9])." << endl;
+		usage (s, 1);
+	      }
+	  }
+
 	  cerr << "Warning: using '-m' disables cache support." << endl;
 	  s.use_cache = false;
           break;
@@ -370,6 +410,10 @@ main (int argc, char * const argv [])
 	  s.macros.push_back (string (optarg));
 	  break;
 
+	case 'q':
+	  s.tapset_compile_coverage = true;
+	  break;
+
         case 'h':
           usage (s, 0);
           break;
@@ -394,8 +438,8 @@ main (int argc, char * const argv [])
 
   if (s.last_pass > 4 && release_changed)
     {
-      cerr << ("Warning: changing last pass to 4 since the kernel release"
-	       " has changed.") << endl;
+      if (s.verbose)
+        cerr << "Warning: changing last pass to 4 since cross-compiling" << endl;
       s.last_pass = 4;
     }
 
@@ -429,6 +473,16 @@ main (int argc, char * const argv [])
            << e << endl;
     }
 
+  // Get rid of a few standard environment variables (which might
+  // cause us to do unintended things).
+  rc = unsetenv("IFS") || unsetenv("CDPATH") || unsetenv("ENV")
+      || unsetenv("BASH_ENV");
+  if (rc)
+    {
+      const char* e = strerror (errno);
+      cerr << "unsetenv failed: " << e << endl;
+    }
+
   s.kernel_base_release.assign(s.kernel_release, 0, s.kernel_release.find('-'));
 
   // arguments parsed; get down to business
@@ -531,9 +585,9 @@ main (int argc, char * const argv [])
             rc ++;
           // GLOB_NOMATCH is acceptable
 
-          if (s.verbose>1)
+          if (s.verbose>1 && globbuf.gl_pathc > 0)
             clog << "Searched '" << dir << "', "
-                 << "match count " << globbuf.gl_pathc << endl;
+                 << "found " << globbuf.gl_pathc << endl;
 
           for (unsigned j=0; j<globbuf.gl_pathc; j++)
             {
@@ -745,6 +799,12 @@ pass_5:
   // if (rc) goto cleanup;
 
  cleanup:
+
+  // update the database information
+  if (!rc && s.tapset_compile_coverage) {
+    update_coverage_db(s);
+  }
+
   // Clean up temporary directory.  Obviously, be careful with this.
   if (s.tmpdir == "")
     ; // do nothing
