@@ -1,5 +1,6 @@
 // -*- C++ -*-
-// Copyright (C) 2005 Red Hat Inc.
+// Copyright (C) 2005-2007 Red Hat Inc.
+// Copyright (C) 2007 Bull S.A.S
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -53,9 +54,13 @@ std::ostream& operator << (std::ostream& o, const token& t);
 struct parse_error: public std::runtime_error
 {
   const token* tok;
-  parse_error (const std::string& msg): runtime_error (msg), tok (0) {}
-  parse_error (const std::string& msg, const token* t): runtime_error (msg),
-                                                        tok (t) {}
+  bool skip_some;
+  parse_error (const std::string& msg): 
+    runtime_error (msg), tok (0), skip_some (true) {}
+  parse_error (const std::string& msg, const token* t): 
+    runtime_error (msg), tok (t), skip_some (true) {}
+  parse_error (const std::string& msg, bool skip): 
+    runtime_error (msg), tok (0), skip_some (skip) {}
 };
 
 
@@ -64,7 +69,7 @@ struct systemtap_session;
 class lexer
 {
 public:
-  token* scan ();
+  token* scan (bool expand_args=true);
   lexer (std::istream&, const std::string&, systemtap_session&);
 
 private:
@@ -124,7 +129,7 @@ private:
 
   // preprocessing subordinate
   std::vector<const token*> enqueued_pp;
-  const token* scan_pp ();
+  const token* scan_pp (bool expand_args=true);
 
   // scanning state
   const token* last ();
