@@ -15,19 +15,60 @@
 int out_fd[NR_CPUS];
 static pthread_t reader[NR_CPUS];
 static int relay_fd[NR_CPUS];
-static int stop_threads = 0;
 static int bulkmode = 0;
+static volatile int stop_threads = 0;
+
+/*
+ * ppoll exists in glibc >= 2.4
+ */
+#if (__GLIBC__ < 2) || ((__GLIBC__ == 2) && (__GLIBC_MINOR__ < 4))
+#define NEED_PPOLL
+#endif
+
+#ifdef NEED_PPOLL
+static int ppoll(struct pollfd *fds, nfds_t nfds,
+		 const struct timespec *timeout, const sigset_t *sigmask)
+{
+	sigset_t origmask;
+	int ready;
+	int tim;
+	if (timeout == NULL)
+		tim = -1;
+	else
+		tim = timeout->tv_sec * 1000 + timeout->tv_nsec / 1000000;
+
+	sigprocmask(SIG_SETMASK, sigmask, &origmask);
+	ready = poll(fds, nfds, tim);
+	sigprocmask(SIG_SETMASK, &origmask, NULL);
+	return ready;
+}
+#endif
 
 /**
  *	reader_thread - per-cpu channel buffer reader
  */
+static void empty_handler(int __attribute__((unused)) sig) { /* do nothing */  }
 
 static void *reader_thread(void *data)
 {
         char buf[131072];
         int rc, cpu = (int)(long)data;
         struct pollfd pollfd;
-	int max_rd = 0;
+	struct timespec tim = {.tv_sec=0, .tv_nsec=200000000}, *timeout = &tim;
+	sigset_t sigs;
+	struct sigaction sa;
+
+	sigemptyset(&sigs);
+	sigaddset(&sigs,SIGUSR2);
+	pthread_sigmask(SIG_BLOCK, &sigs, NULL);
+
+	sigfillset(&sigs);
+	sigdelset(&sigs,SIGUSR2);
+	
+	sa.sa_handler = empty_handler;
+        sa.sa_flags = 0;
+	sigemptyset(&sa.sa_mask);
+        sigaction(SIGUSR2, &sa, NULL);
 
 	if (bulkmode) {
 		cpu_set_t cpu_mask;
@@ -36,42 +77,39 @@ static void *reader_thread(void *data)
 		if( sched_setaffinity( 0, sizeof(cpu_mask), &cpu_mask ) < 0 ) {
 			perror("sched_setaffinity");
 		}
+#ifdef NEED_PPOLL
+		/* Without a real ppoll, there is a small race condition that could */
+		/* block ppoll(). So use a timeout to prevent that. */
+		timeout->tv_sec = 10;
+		timeout->tv_nsec = 0;
+#else
+		timeout = NULL;
+#endif
 	}
 	
 	pollfd.fd = relay_fd[cpu];
 	pollfd.events = POLLIN;
 
         do {
-                rc = poll(&pollfd, 1, 10);
+                rc = ppoll(&pollfd, 1, timeout, &sigs);
                 if (rc < 0) {
+			dbug(3, "cpu=%d poll=%d errno=%d\n", cpu, rc, errno);
                         if (errno != EINTR) {
 				fprintf(stderr, "poll error: %s\n",strerror(errno));
-				pthread_exit(NULL);
+				return(NULL);
                         }
-                        fprintf(stderr, "poll warning: %s\n",strerror(errno));
+			stop_threads = 1;
                 }
-                rc = read(relay_fd[cpu], buf, sizeof(buf));
-                if (!rc) {
-                        continue;
+		while ((rc = read(relay_fd[cpu], buf, sizeof(buf))) > 0) {
+			if (write(out_fd[cpu], buf, rc) != rc) {
+				fprintf(stderr, "Couldn't write to output fd %d for cpu %d, exiting: errcode = %d: %s\n", 
+					out_fd[cpu], cpu, errno, strerror(errno));
+				return(NULL);
+			}
 		}
-                if (rc < 0) {
-                        if (errno == EAGAIN)
-                                continue;
-			fprintf(stderr, "error reading fd %d on cpu %d: %s\n", relay_fd[cpu], cpu, strerror(errno));
-			continue;
-                }
-
-		if (rc > max_rd)
-			max_rd = rc;
-
-                if (write(out_fd[cpu], buf, rc) < 0) {
-			fprintf(stderr, "Couldn't write to output fd %d for cpu %d, exiting: errcode = %d: %s\n", 
-				out_fd[cpu], cpu, errno, strerror(errno));
-			pthread_exit(NULL);
-                }
-
         } while (!stop_threads);
-	pthread_exit((void *)(long)max_rd);
+	dbug(3, "exiting thread %d\n", cpu);
+	return(NULL);
 }
 
 /**
@@ -79,36 +117,50 @@ static void *reader_thread(void *data)
  *
  *	Returns 0 if successful, negative otherwise
  */
-int init_relayfs(struct _stp_msg_trans *t)
+int init_relayfs(void)
 {
 	int i;
 	struct statfs st;
 	char buf[128], relay_filebase[128];
 
-	bulkmode = t->bulk_mode;
-	dbug("initializing relayfs. bulkmode = %d\n", bulkmode);
+	dbug(1, "initializing relayfs\n");
 
 	reader[0] = (pthread_t)0;
 	relay_fd[0] = 0;
 	out_fd[0] = 0;
 
  	if (statfs("/sys/kernel/debug", &st) == 0 && (int) st.f_type == (int) DEBUGFS_MAGIC)
- 		sprintf(relay_filebase, "/sys/kernel/debug/systemtap_%d", getpid());
+ 		sprintf(relay_filebase, "/sys/kernel/debug/systemtap/%s", modname);
  	else {
 		fprintf(stderr,"Cannot find relayfs or debugfs mount point.\n");
 		return -1;
 	}
 
+	if (send_request(STP_BULK, buf, sizeof(buf)) > 0)
+		bulkmode = 1;
+
+	for (i = 0; i < NR_CPUS; i++) {
+		sprintf(buf, "%s/trace%d", relay_filebase, i);
+		dbug(2, "attempting to open %s\n", buf);
+		relay_fd[i] = open(buf, O_RDONLY | O_NONBLOCK);
+		if (relay_fd[i] < 0)
+			break;
+	}
+	ncpus = i;
+	dbug(2, "ncpus=%d, bulkmode = %d\n", ncpus, bulkmode);
+
+	if (ncpus == 0) {
+		err("couldn't open %s.\n", buf);
+		return -1;
+	}
+	if (ncpus > 1 && bulkmode == 0) {
+		err("ncpus=%d, bulkmode = %d\n", ncpus, bulkmode);
+		err("This is inconsistent! Please file a bug report. Exiting now.\n");
+		return -1;
+	}
+
 	if (bulkmode) {
 		for (i = 0; i < ncpus; i++) {
-			sprintf(buf, "%s/trace%d", relay_filebase, i);
-			dbug("opening %s\n", buf);
-			relay_fd[i] = open(buf, O_RDONLY | O_NONBLOCK);
-			if (relay_fd[i] < 0) {
-				fprintf(stderr, "ERROR: couldn't open relayfs file %s.\n", buf);
-				return -1;
-			}
-			
 			if (outfile_name) {
 				/* special case: for testing we sometimes want to write to /dev/null */
 				if (strcmp(outfile_name, "/dev/null") == 0)
@@ -117,27 +169,15 @@ int init_relayfs(struct _stp_msg_trans *t)
 					sprintf(buf, "%s_%d", outfile_name, i);
 			} else
 				sprintf(buf, "stpd_cpu%d", i);
-
+			
 			out_fd[i] = open (buf, O_CREAT|O_TRUNC|O_WRONLY, 0666);
-			dbug("out_fd[%d] = %d\n", i, out_fd[i]);
 			if (out_fd[i] < 0) {
 				fprintf(stderr, "ERROR: couldn't open output file %s.\n", buf);
 				return -1;
 			}
 		}
-		
 	} else {
 		/* stream mode */
-		ncpus = 1;
-		sprintf(buf, "%s/trace0", relay_filebase);
-		dbug("opening %s\n", buf);
-		relay_fd[0] = open(buf, O_RDONLY | O_NONBLOCK);
-		dbug("got fd=%d\n", relay_fd[0]);
-		if (relay_fd[0] < 0) {
-			fprintf(stderr, "ERROR: couldn't open relayfs file %s.\n", buf);
-			return -1;
-		}
-
 		if (outfile_name) {
 			out_fd[0] = open (outfile_name, O_CREAT|O_TRUNC|O_WRONLY, 0666);
 			if (out_fd[0] < 0) {
@@ -146,9 +186,9 @@ int init_relayfs(struct _stp_msg_trans *t)
 			}
 		} else
 			out_fd[0] = STDOUT_FILENO;
-
+		
 	}
-	dbug("starting threads\n");
+	dbug(2, "starting threads\n");
 	for (i = 0; i < ncpus; i++) {
 		if (pthread_create(&reader[i], NULL, reader_thread, (void *)(long)i) < 0) {
 			fprintf(stderr, "failed to create thread\n");
@@ -163,24 +203,26 @@ int init_relayfs(struct _stp_msg_trans *t)
 void close_relayfs(void)
 {
 	int i;
-	void *res;
-	dbug("closing\n");
-
 	stop_threads = 1;
-
+	dbug(2, "closing\n");
 	for (i = 0; i < ncpus; i++) {
 		if (reader[i]) 
-			pthread_join(reader[i], &res);
+			pthread_kill(reader[i], SIGUSR2);
+		else
+			break;
+	}
+	for (i = 0; i < ncpus; i++) {
+		if (reader[i])
+			pthread_join(reader[i], NULL);
 		else
 			break;
 	}
-
 	for (i = 0; i < ncpus; i++) {
 		if (relay_fd[i] >= 0)
 			close(relay_fd[i]);
 		else
 			break;
 	}
-	dbug("closed files\n");
+	dbug(2, "done\n");
 }
 
