@@ -9,7 +9,7 @@
  * later version.
  */
 
-#define STP_DEFAULT_BUFFERS 20
+#define STP_DEFAULT_BUFFERS 50
 static int _stp_current_buffers = STP_DEFAULT_BUFFERS;
 
 static struct list_head _stp_ready_q;
@@ -59,6 +59,12 @@ static ssize_t _stp_ctl_write_cmd (struct file *file, const char __user *buf,
 	case STP_EXIT:
 		_stp_exit_flag = 1;
 		break;
+	case STP_BULK:
+#ifdef STP_BULKMODE
+		return count;
+#else
+		return -1;
+#endif
 	default:
 		errk ("invalid command type %d\n", type);
 		return -EINVAL;
@@ -71,7 +77,7 @@ struct _stp_buffer {
 	struct list_head list;
 	int len;
 	int type;
-	char buf[STP_BUFFER_SIZE];
+	char buf[256];
 };
 
 static DECLARE_WAIT_QUEUE_HEAD(_stp_ctl_wq);
@@ -111,7 +117,7 @@ static void _stp_ctl_write_dbug (int type, void *data, int len)
 }
 #endif
 
-static int _stp_ctl_write (int type, void *data, int len)
+static int _stp_ctl_write (int type, void *data, unsigned len)
 {
 	struct _stp_buffer *bptr;
 	unsigned long flags;
@@ -120,14 +126,16 @@ static int _stp_ctl_write (int type, void *data, int len)
 #ifdef DEBUG
 	_stp_ctl_write_dbug(type, data, len);
 #endif
+
 	numtrylock = 0;
 	while (!spin_trylock_irqsave (&_stp_pool_lock, flags) && (++numtrylock < MAXTRYLOCK)) 
 		ndelay (TRYLOCKDELAY);
-	if (unlikely (numtrylock >= MAXTRYLOCK)) 
+	if (unlikely (numtrylock >= MAXTRYLOCK))
 		return 0;
 
-	if (list_empty(&_stp_pool_q)) {
+	if (unlikely(list_empty(&_stp_pool_q))) {
 		spin_unlock_irqrestore(&_stp_pool_lock, flags); 
+		dbug("_stp_pool_q empty\n");
 		return -1;
 	}
 
@@ -137,15 +145,16 @@ static int _stp_ctl_write (int type, void *data, int len)
 	spin_unlock_irqrestore(&_stp_pool_lock, flags);
 
 	bptr->type = type;
-	memcpy (bptr->buf, data, len);
+	memcpy(bptr->buf, data, min((size_t) len, sizeof(bptr->buf)));
 	bptr->len = len;
 	
 	/* put it on the pool of ready buffers */
 	numtrylock = 0;
 	while (!spin_trylock_irqsave (&_stp_ready_lock, flags) && (++numtrylock < MAXTRYLOCK)) 
 		ndelay (TRYLOCKDELAY);
-	if (unlikely (numtrylock >= MAXTRYLOCK))
+	if (unlikely (numtrylock >= MAXTRYLOCK)) 
 		return 0;
+
 	list_add_tail(&bptr->list, &_stp_ready_q);
 	spin_unlock_irqrestore(&_stp_ready_lock, flags);
 
@@ -206,11 +215,25 @@ _stp_ctl_read_cmd (struct file *file, char __user *buf, size_t count, loff_t *pp
 	return len;
 }
 
+static int _stp_ctl_open_cmd (struct inode *inode, struct file *file)
+{
+	_stp_pid = current->pid;
+	return 0;
+}
+
+static int _stp_ctl_close_cmd (struct inode *inode, struct file *file)
+{
+	_stp_pid = 0;
+	return 0;
+
+}
 
 static struct file_operations _stp_ctl_fops_cmd = {
 	.owner = THIS_MODULE,
 	.read = _stp_ctl_read_cmd,
 	.write = _stp_ctl_write_cmd,
+	.open = _stp_ctl_open_cmd,
+	.release = _stp_ctl_close_cmd,
 };
 
 static struct dentry *_stp_cmd_file = NULL;
@@ -240,11 +263,10 @@ static int _stp_register_ctl_channel (void)
 	}
 	
 
-	/* create [debugfs]/systemtap_[pid]/cmd  */
-	_stp_cmd_file = debugfs_create_file("cmd", 0444, _stp_utt->utt_tree_root, NULL, &_stp_ctl_fops_cmd);
+	/* create [debugfs]/systemtap/module_name/cmd  */
+	_stp_cmd_file = debugfs_create_file("cmd", 0600, _stp_utt->dir, NULL, &_stp_ctl_fops_cmd);
 	if (_stp_cmd_file == NULL) 
 		goto err0;
-
 	return 0;
 
 err0:
