@@ -29,6 +29,7 @@ struct functiondecl;
 struct derived_probe;
 struct be_derived_probe_group;
 struct dwarf_derived_probe_group;
+struct uprobe_derived_probe_group;
 struct timer_derived_probe_group;
 struct profile_derived_probe_group;
 struct mark_derived_probe_group;
@@ -69,6 +70,7 @@ struct systemtap_session
 {
   systemtap_session ();
   // NB: new POD members likely need to be explicitly cleared in the ctor.
+  // See elaborate.cxx.
 
   // command line args
   std::vector<std::string> include_path;
@@ -95,6 +97,7 @@ struct systemtap_session
   unsigned perfmon;
   bool symtab; /* true: emit symbol table at translation time; false: let staprun do it. */
   bool prologue_searching;
+  bool tapset_compile_coverage;
 
   // Cache data
   bool use_cache;
@@ -120,12 +123,17 @@ struct systemtap_session
   std::vector<derived_probe*> probes; // see also *_probes groups below
   std::vector<embeddedcode*> embeds;
   std::map<std::string, statistic_decl> stat_decls;
+  // track things that are removed
+  std::vector<vardecl*> unused_globals;
+  std::vector<derived_probe*> unused_probes; // see also *_probes groups below
+  std::vector<functiondecl*> unused_functions;
   // XXX: vector<*> instead please?
 
   // Every probe in these groups must also appear in the
   // session.probes vector.
   be_derived_probe_group* be_derived_probes;
   dwarf_derived_probe_group* dwarf_derived_probes;
+  uprobe_derived_probe_group* uprobe_derived_probes;
   timer_derived_probe_group* timer_derived_probes;
   profile_derived_probe_group* profile_derived_probes;
   mark_derived_probe_group* mark_derived_probes;
