@@ -15,6 +15,8 @@
 #include <typeinfo>
 #include <sstream>
 #include <cassert>
+#include <vector>
+#include <algorithm>
 
 using namespace std;
 
@@ -847,6 +849,13 @@ void probe::printsig (ostream& o) const
 }
 
 
+void
+probe::collect_derivation_chain (std::vector<derived_probe*> &probes_list)
+{
+  probes_list.push_back((derived_probe*)this);
+}
+
+
 void probe_point::print (ostream& o) const
 {
   for (unsigned i=0; i<components.size(); i++)
@@ -861,6 +870,22 @@ void probe_point::print (ostream& o) const
     o << "?";
 }
 
+string probe_point::str ()
+{
+  ostringstream o;
+  for (unsigned i=0; i<components.size(); i++)
+    {
+      if (i>0) o << ".";
+      probe_point::component* c = components[i];
+      o << c->functor;
+      if (c->arg)
+        o << "(" << *c->arg << ")";
+    }
+  if (optional)
+    o << "?";
+  return o.str();
+}
+
 
 probe_alias::probe_alias(std::vector<probe_point*> const & aliases):
   probe (), alias_names (aliases)
@@ -1670,6 +1695,34 @@ varuse_collecting_visitor::visit_delete_statement (delete_statement* s)
   current_lrvalue = last_lrvalue;
 }
 
+bool
+varuse_collecting_visitor::side_effect_free ()
+{
+  return (written.empty() && !embedded_seen);
+}
+
+
+bool
+varuse_collecting_visitor::side_effect_free_wrt (const set<vardecl*>& vars)
+{
+  // A looser notion of side-effect-freeness with respect to a given
+  // list of variables.
+
+  // That's useful because the written list may consist of local
+  // variables of called functions.  But visible side-effects only
+  // occur if the client's locals, or any globals are written-to.
+
+  set<vardecl*> intersection;
+  insert_iterator<set<vardecl*> > int_it (intersection, intersection.begin());
+  set_intersection (written.begin(), written.end(),
+                    vars.begin(), vars.end(),
+                    int_it);
+
+  return (intersection.empty() && !embedded_seen);
+}
+
+
+
 
 // ------------------------------------------------------------------------
 
